const HEADER_SPY_PREF_SERVICE = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefService);

const HEADER_SPY_PREF_BRANCH = HEADER_SPY_PREF_SERVICE.getBranch("extensions.header_spy.");

const HEADER_SPY_OBSERV_SERVICE = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);

// ---


// ---

var header_spy_options_store = {
    defaults : {
        header          : "Server",
        headersrc       : 1,
        
        max_width       : 250,
        show_if_exist   : true,
        show_name       : false,
        popup_headers   : {
            request         : "",//"Host REQUEST User-Agent Cookie",
            response        : "",//"RESPONSE Date Set-Cookie",
            show_if_exist   : true,
            wrap_on         : 100,
            in_tooltip      : true,
            in_menu         : true
            },

        show_tools_menu_item : false,
        expand_cookie_in_menu : true,
        
        status           : "* none *",
        max_expand_width : 6666,
        max_panels       : 5,
        sbp_name_prefix  : "header-spy-sbp",
        sbp_tooltip_prefix : "header-spy-sbp-tooltip",
        sbp_menu_prefix  : "header-spy-sbp-menu"
    },

    get_int: function(name, default_value) {
        var value;
        try {
            value = HEADER_SPY_PREF_BRANCH.getIntPref(name);
        }
        catch (e) {
            value = default_value;
        }
        return value;
    },

    get_bool: function(name, default_value) {
        var value;
        try {
            value = HEADER_SPY_PREF_BRANCH.getBoolPref(name);
        }
        catch (e) {
            value = default_value;
        }
        return value;
    },

    get_char: function(name, default_value) {
        var value;
        try {
            value = HEADER_SPY_PREF_BRANCH.getCharPref(name);
        }
        catch (e) {
            value = default_value;
        }
        return value;
    },

    // ---

    get_rec: function(i) {
        var options_panel = {
            header          : this.defaults.header,
            headersrc       : this.defaults.headersrc, // 0 - Request, 1 - Response
            max_width       : this.defaults.max_width,
            show_if_exist   : this.defaults.show_if_exist,
            show_name       : this.defaults.show_name,
            popup_headers   : {
                request         : this.defaults.popup_headers.request,
                response        : this.defaults.popup_headers.response,
                show_if_exist   : this.defaults.popup_headers.show_if_exist,
                wrap_on         : this.defaults.popup_headers.wrap_on,
                in_tooltip      : this.defaults.popup_headers.in_tooltip,
                in_menu         : this.defaults.popup_headers.in_menu,
                },
            
            current_status  : this.defaults.default_status,
            sbp_name        : header_spy_options_store.defaults.sbp_name_prefix + i,
            sbp_tooltip     : header_spy_options_store.defaults.sbp_tooltip_prefix + i,
            sbp_menu        : header_spy_options_store.defaults.sbp_menu_prefix + i
            };                

        return options_panel;
    },

    get: function() {
        var options = {
            show_tools_menu_item : null,
            expand_cookie_in_menu : null,
            panels : []
            };
        
        options.show_tools_menu_item = this.get_bool("show_tools_menu_item", this.defaults.show_tools_menu_item);
        options.expand_cookie_in_menu = this.get_bool("expand_cookie_in_menu", this.defaults.expand_cookie_in_menu);
        
        for (i = 0 ; ; i++) {
            s_header = this.get_char("header" + i, "");
            if (s_header == "") break; // if not exists or empty

            // ---
            
            p = this.get_rec(i);
            
            p.header                        = s_header;
            p.headersrc                     = this.get_int( "headersrc" + i,                   p.headersrc);
                                           
            var mw = this.get_int( "max_width" + i, p.max_width);
            if (mw > 0) p.max_width = mw; // Don't use values <= 0
            p.show_if_exist                 = this.get_bool("show_if_exist" + i,               p.show_if_exist);
            p.show_name                     = this.get_bool("show_name" + i,                   p.show_name);
                                           
            p.popup_headers.request         = this.get_char("popup_headers.request" + i,       p.popup_headers.request);
            p.popup_headers.response        = this.get_char("popup_headers.response" + i,      p.popup_headers.response);
            p.popup_headers.show_if_exist   = this.get_bool("popup_headers.show_if_exist" + i, p.popup_headers.show_if_exist);
            p.popup_headers.wrap_on         = this.get_int( "popup_headers.wrap_on" + i,       p.popup_headers.wrap_on);
            p.popup_headers.in_tooltip      = this.get_bool("popup_headers.in_tooltip" + i,    p.popup_headers.in_tooltip);
            p.popup_headers.in_menu         = this.get_bool("popup_headers.in_menu" + i,       p.popup_headers.in_menu);

            options.panels.push(p);
        }

        return options;
    },

    save: function(options) {
        var original_options = header_spy_options_store.get();

        HEADER_SPY_PREF_BRANCH.setBoolPref("show_tools_menu_item", options.show_tools_menu_item);
        HEADER_SPY_PREF_BRANCH.setBoolPref("expand_cookie_in_menu", options.expand_cookie_in_menu);
        
        for (i = 0 ; i < options.panels.length ; i++) {
            var p = options.panels[i];

            HEADER_SPY_PREF_BRANCH.setCharPref("header" + i,                      p.header);
            HEADER_SPY_PREF_BRANCH.setIntPref( "headersrc" + i,                   p.headersrc);
                                                                                  
            HEADER_SPY_PREF_BRANCH.setIntPref( "max_width" + i,                   p.max_width);
            HEADER_SPY_PREF_BRANCH.setBoolPref("show_if_exist" + i,               p.show_if_exist);
            HEADER_SPY_PREF_BRANCH.setBoolPref("show_name" + i,                   p.show_name);
                                                                                     
            HEADER_SPY_PREF_BRANCH.setCharPref("popup_headers.request" + i,       p.popup_headers.request);       
            HEADER_SPY_PREF_BRANCH.setCharPref("popup_headers.response" + i,      p.popup_headers.response);      
            HEADER_SPY_PREF_BRANCH.setBoolPref("popup_headers.show_if_exist" + i, p.popup_headers.show_if_exist);
            HEADER_SPY_PREF_BRANCH.setIntPref( "popup_headers.wrap_on" + i,       p.popup_headers.wrap_on);        
            HEADER_SPY_PREF_BRANCH.setBoolPref("popup_headers.in_tooltip" + i,    p.popup_headers.in_tooltip);
            HEADER_SPY_PREF_BRANCH.setBoolPref("popup_headers.in_menu" + i,       p.popup_headers.in_menu);
        }

        for ( ; i < original_options.panels.length ; i++) { // clear previous options
            HEADER_SPY_PREF_BRANCH.clearUserPref("header" + i);
        }

        HEADER_SPY_OBSERV_SERVICE.notifyObservers(null, "header-spy-load-prefs", false);
    }
};

// ---
